@@ -1,4 +1,4 @@
-/* $Id: table.h,v 1.5 2001/11/30 10:10:24 ukai Exp $ */
+/* $Id: table.h,v 1.5.6.2 2002/11/26 07:10:18 ukai Exp $ */
 #if (defined(MESCHACH) && !defined(MATRIX))
 #define MATRIX
 #endif				/* (defined(MESCHACH) && !defined(MATRIX)) */
@@ -34,14 +34,15 @@ typedef unsigned short table_attr;
 #define TBL_IN_COL     4
 
 #define MAXCELL 20
+#define MAXROWCELL 1000
 struct table_cell {
     short col[MAXCELL];
     short colspan[MAXCELL];
-    char index[MAXCELL];
+    short index[MAXCELL];
     short maxcell;
     short icell;
 #ifdef MATRIX
-    char eindex[MAXCELL];
+    short eindex[MAXCELL];
     short necell;
 #endif				/* MATRIX */
     short width[MAXCELL];
